#include <linux/list.h>
#include <linux/device.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <linux/sysdev.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>

#include <asm/mach/arch.h>
#include <asm/mach/irq.h>

#include "BuffaloGpio.h"
#include "mvGpp.h"
#include "mvGppRegs.h"

#include "mvDS1339.h"
#include "mvDS1339Reg.h"

#define MagicKeyAPC	0x45
#define MagicKeyOMR	0x3a
#define MagicKeyUSB	0x16

/*
	GPIO-setup
	LS-GL	BIT2 : micon-int
			BIT3 : RTC-int
	TS-HTGL
			BIT8 : micon-int
			BIT9 : RTC-int
			BIT13: UPS-SerailPort
			BIT14: OMRON_UPS Battery Low detection
*/

#define BIT(x) (1<<x)

//----------------------------------------------------------------------
// micon
//----------------------------------------------------------------------
void BuffaloGpio_MiconIntSetup(void)
{
	mvGppOutEnablle(0, BIT(BIT_MICON) , BIT(BIT_MICON));		// disable output
	mvGppPolaritySet(MICON_POL, BIT(BIT_MICON) , BIT(BIT_MICON));
	// set GPIO-2 : edge IRQ
	///set_irq_handler(32+BIT_MICON, do_edge_IRQ);
}

//----------------------------------------------------------------------
void BuffaloGpio_ClearMiconInt(void)
{
	unsigned cause;
	//printk(">%s\n",__FUNCTION__);
	/*
	printk("MPP_CONTROL_REG  %x %x %x %x\n"
		,MV_REG_READ(mvMppRegGet(0))
		,MV_REG_READ(mvMppRegGet(1))
		,MV_REG_READ(mvMppRegGet(2))
		,MV_REG_READ(mvMppRegGet(3))
		);
	*/
	
	cause = MV_REG_READ(GPP_INT_CAUSE_REG(0));
	//printk("cause:%x\n",cause);
	MV_REG_WRITE(GPP_INT_CAUSE_REG(0), ~(cause & BIT(BIT_MICON)));
}

//----------------------------------------------------------------------
// RTC
//----------------------------------------------------------------------
void BuffaloGpio_RtcIntSetup(void)
{
	mvGppOutEnablle(0, BIT(BIT_RTC) , BIT(BIT_RTC));		// disable output
	mvGppPolaritySet(0, BIT(BIT_RTC) , 0);
	MV_REG_WRITE(GPP_INT_LVL_REG(0), 0);
}

//----------------------------------------------------------------------
void BuffaloGpio_ClearRtcInt(void)
{
	unsigned cause;
	
	cause = MV_REG_READ(GPP_INT_CAUSE_REG(0));
	MV_REG_WRITE(GPP_INT_CAUSE_REG(0), ~(cause & BIT(BIT_RTC)));
}


#ifdef CONFIG_BUFFALO_USE_UPS
// for ups codes
void BuffaloGpio_UPSPortEnable(void){

	mvGppOutEnablle(0, BIT(BIT_UPS), (MV_GPP_OUT_EN & BIT(BIT_UPS)));
	mvGppValueSet(0, BIT(BIT_UPS), BIT(BIT_UPS));

}

void BuffaloGpio_UPSPortDisable(void){

	mvGppOutEnablle(0, BIT(BIT_UPS), (MV_GPP_OUT_DIS & BIT(BIT_UPS)));
	mvGppValueSet(0, BIT(BIT_UPS), (0 & BIT(BIT_UPS)));

}

unsigned int BuffaloGpio_UPSPortScan(void){

	MV_U32 ret = mvGppValueGet(0, BIT(BIT_UPS));
	return (unsigned int)( ret & BIT(BIT_UPS));
	
}
// end of basically ups codes

////////////////////////////////////////////////
void BuffaloGpio_UPSOmronBLEnable(void){
	
	mvGppOutEnablle(0, BIT(BIT_OMR_BL), (MV_GPP_OUT_DIS & BIT(BIT_OMR_BL)));
	
}

void BuffaloGpio_UPSOmronBLDisable(void){
	
	mvGppOutEnablle(0, BIT(BIT_OMR_BL), (MV_GPP_OUT_EN & BIT(BIT_OMR_BL)));
	mvGppValueSet(0, BIT(BIT_OMR_BL), (0 & BIT(BIT_OMR_BL)));
	
}

unsigned int BuffaloGpio_UPSOmronBLGetStatus(void){
	
	MV_U32 ret = mvGppValueGet(0, BIT(BIT_OMR_BL));
	return (unsigned int)( ret & BIT(BIT_OMR_BL));
	
}

unsigned int BuffaloGpio_UPSOmronBLUseStatus(void){

	unsigned base=0xF1000000;								////caution base addr directry using!!!!///////
	volatile unsigned *gpioStat=(unsigned *)(base+GPP_DATA_OUT_EN_REG(0));
//	printk("0x%x\n", gpio[1]);
	return (gpioStat[0] & BIT(BIT_OMR_BL));
	
}
///////////////////////////////////////////////////////


// for ups recover function

void BuffaloRtc_UPSRecoverInit(void){
	mvRtcDS1339Init();
	BufRtcDS1339AlarmBSet(0);
}

void BuffaloRtc_UPSRecoverEnable(int8_t TargetType){

	int8_t MagicKey=0;

	switch(TargetType){
	case RECOVER_TARGET_UPS_APC:
		MagicKey = MagicKeyAPC;
		break;
	case RECOVER_TARGET_UPS_OMR:
		MagicKey = MagicKeyOMR;
		break;
	case RECOVER_TARGET_UPS_USB: 
		MagicKey = MagicKeyUSB;
		break;
	default:
		break;
	}
	
	if(MagicKey){
		BufRtcDS1339AlarmBSet(MagicKey);
	}
	
}

void BuffaloRtc_UPSRecoverDisable(void){
	
	BufRtcDS1339AlarmBSet(0);
}

int BuffaloRtc_UPSRecoverReadStatus(void){
	
	int8_t data;
	BufRtcDS1339AlarmBGet(&data);

	switch(data){
	case MagicKeyAPC:
		return RECOVER_TARGET_UPS_APC;
		break;
	case MagicKeyOMR:
		return RECOVER_TARGET_UPS_OMR;
		break;
	case MagicKeyUSB:
		return RECOVER_TARGET_UPS_USB;
		break;
	default:
		break;
	}
	return -1;
}
// end of recover function 


////
// for debug
////
/*
unsigned int BuffaloGpio_PortScan(void){
	
	MV_U32 ret = mvGppValueGet(0, 0xffff);
	
	return ret;
}*/

#endif  //CONFIG_BUFFALO_USE_UPS

//----------------------------------------------------------------------
// for Debug
//----------------------------------------------------------------------
/*
void BuffaloPrintGpio(void)
{
	printk("GPIO:Din=%x Int=%x mask=%x lmask=%x\n "
		,MV_REG_READ(GPP_DATA_IN_REG(0))
		,MV_REG_READ(GPP_INT_CAUSE_REG(0))
		,MV_REG_READ(GPP_INT_MASK_REG(0))
		,MV_REG_READ(GPP_INT_LVL_REG(0))
		);
}
*/

